@@ -97,10 +97,6 @@ source_suffix = ".txt"
 # The root toctree document.
 root_doc = "contents"
 
-# Disable auto-created table of contents entries for all domain objects (e.g.
-# functions, classes, attributes, etc.) in Sphinx 5.2+.
-toc_object_entries = False
-
 # General substitutions.
 project = "Django"
 copyright = "Django Software Foundation and contributors"
@@ -111,7 +107,7 @@ copyright = "Django Software Foundation and contributors"
 # built documents.
 #
 # The short X.Y version.
-version = "4.0"
+version = "4.1"
 # The full version, including alpha/beta/rc tags.
 try:
     from django import VERSION, get_version
@@ -128,7 +124,7 @@ else:
     release = django_release()
 
 # The "development version" of Django
-django_next_version = "4.1"
+django_next_version = "4.2"
 
 extlinks = {
     "bpo": ("https://bugs.python.org/issue?@action=redirect&bpo=%s", "bpo-%s"),
@@ -277,13 +273,23 @@ rst_epilog = """
 
 # -- Options for LaTeX output --------------------------------------------------
 
+# Use XeLaTeX for Unicode support.
+latex_engine = "xelatex"
+latex_use_xindy = False
+# Set font for CJK and fallbacks for unicode characters.
 latex_elements = {
-    "preamble": (
-        "\\DeclareUnicodeCharacter{2264}{\\ensuremath{\\le}}"
-        "\\DeclareUnicodeCharacter{2265}{\\ensuremath{\\ge}}"
-        "\\DeclareUnicodeCharacter{2665}{[unicode-heart]}"
-        "\\DeclareUnicodeCharacter{2713}{[unicode-checkmark]}"
-    ),
+    "fontpkg": r"""
+        \setmainfont{Symbola}
+    """,
+    "preamble": r"""
+        \usepackage{newunicodechar}
+        \usepackage[UTF8]{ctex}
+        \newunicodechar{π}{\ensuremath{\pi}}
+        \newunicodechar{≤}{\ensuremath{\le}}
+        \newunicodechar{≥}{\ensuremath{\ge}}
+        \newunicodechar{♥}{\ensuremath{\heartsuit}}
+        \newunicodechar{…}{\ensuremath{\ldots}}
+    """,
 }
 
 # Grouping the document tree into LaTeX files. List of tuples
