@@ -21,6 +21,7 @@ from django.db.models import (
     CharField,
     DateField,
     DateTimeField,
+    ForeignKey,
     ManyToManyField,
     UUIDField,
 )
@@ -42,6 +43,7 @@ from .models import (
     Member,
     MyFileField,
     Profile,
+    ReleaseEvent,
     School,
     Student,
     UnsafeLimitChoicesTo,
@@ -140,6 +142,17 @@ class AdminFormfieldForDBFieldTests(SimpleTestCase):
         )
         self.assertIsNone(ff.empty_label)
 
+    def test_radio_fields_foreignkey_formfield_overrides_empty_label(self):
+        class MyModelAdmin(admin.ModelAdmin):
+            radio_fields = {"parent": admin.VERTICAL}
+            formfield_overrides = {
+                ForeignKey: {"empty_label": "Custom empty label"},
+            }
+
+        ma = MyModelAdmin(Inventory, admin.site)
+        ff = ma.formfield_for_dbfield(Inventory._meta.get_field("parent"), request=None)
+        self.assertEqual(ff.empty_label, "Custom empty label")
+
     def test_many_to_many(self):
         self.assertFormfield(Band, "members", forms.SelectMultiple)
 
@@ -613,19 +626,27 @@ class ForeignKeyRawIdWidgetTest(TestCase):
         band.album_set.create(
             name="Hybrid Theory", cover_art=r"albums\hybrid_theory.jpg"
         )
-        rel = Album._meta.get_field("band").remote_field
-
-        w = widgets.ForeignKeyRawIdWidget(rel, widget_admin_site)
+        rel_uuid = Album._meta.get_field("band").remote_field
+        w = widgets.ForeignKeyRawIdWidget(rel_uuid, widget_admin_site)
         self.assertHTMLEqual(
             w.render("test", band.uuid, attrs={}),
             '<input type="text" name="test" value="%(banduuid)s" '
-            'class="vForeignKeyRawIdAdminField">'
+            'class="vForeignKeyRawIdAdminField vUUIDField">'
             '<a href="/admin_widgets/band/?_to_field=uuid" class="related-lookup" '
             'id="lookup_id_test" title="Lookup"></a>&nbsp;<strong>'
             '<a href="/admin_widgets/band/%(bandpk)s/change/">Linkin Park</a>'
             "</strong>" % {"banduuid": band.uuid, "bandpk": band.pk},
         )
 
+        rel_id = ReleaseEvent._meta.get_field("album").remote_field
+        w = widgets.ForeignKeyRawIdWidget(rel_id, widget_admin_site)
+        self.assertHTMLEqual(
+            w.render("test", None, attrs={}),
+            '<input type="text" name="test" class="vForeignKeyRawIdAdminField">'
+            '<a href="/admin_widgets/album/?_to_field=id" class="related-lookup" '
+            'id="lookup_id_test" title="Lookup"></a>',
+        )
+
     def test_relations_to_non_primary_key(self):
         # ForeignKeyRawIdWidget works with fields which aren't related to
         # the model's primary key.
@@ -880,6 +901,7 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
         Pressing the ESC key or clicking on a widget value closes the date and
         time picker widgets.
         """
+        from selenium.webdriver.common.by import By
         from selenium.webdriver.common.keys import Keys
 
         self.admin_login(username="super", password="secret", login_url="/")
@@ -889,59 +911,61 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
         )
 
         # First, with the date picker widget ---------------------------------
-        cal_icon = self.selenium.find_element_by_id("calendarlink0")
+        cal_icon = self.selenium.find_element(By.ID, "calendarlink0")
         # The date picker is hidden
         self.assertFalse(
-            self.selenium.find_element_by_id("calendarbox0").is_displayed()
+            self.selenium.find_element(By.ID, "calendarbox0").is_displayed()
         )
         # Click the calendar icon
         cal_icon.click()
         # The date picker is visible
-        self.assertTrue(self.selenium.find_element_by_id("calendarbox0").is_displayed())
+        self.assertTrue(
+            self.selenium.find_element(By.ID, "calendarbox0").is_displayed()
+        )
         # Press the ESC key
-        self.selenium.find_element_by_tag_name("body").send_keys([Keys.ESCAPE])
+        self.selenium.find_element(By.TAG_NAME, "body").send_keys([Keys.ESCAPE])
         # The date picker is hidden again
         self.assertFalse(
-            self.selenium.find_element_by_id("calendarbox0").is_displayed()
+            self.selenium.find_element(By.ID, "calendarbox0").is_displayed()
         )
         # Click the calendar icon, then on the 15th of current month
         cal_icon.click()
-        self.selenium.find_element_by_xpath("//a[contains(text(), '15')]").click()
+        self.selenium.find_element(By.XPATH, "//a[contains(text(), '15')]").click()
         self.assertFalse(
-            self.selenium.find_element_by_id("calendarbox0").is_displayed()
+            self.selenium.find_element(By.ID, "calendarbox0").is_displayed()
         )
         self.assertEqual(
-            self.selenium.find_element_by_id("id_birthdate_0").get_attribute("value"),
+            self.selenium.find_element(By.ID, "id_birthdate_0").get_attribute("value"),
             datetime.today().strftime("%Y-%m-") + "15",
         )
 
         # Then, with the time picker widget ----------------------------------
-        time_icon = self.selenium.find_element_by_id("clocklink0")
+        time_icon = self.selenium.find_element(By.ID, "clocklink0")
         # The time picker is hidden
-        self.assertFalse(self.selenium.find_element_by_id("clockbox0").is_displayed())
+        self.assertFalse(self.selenium.find_element(By.ID, "clockbox0").is_displayed())
         # Click the time icon
         time_icon.click()
         # The time picker is visible
-        self.assertTrue(self.selenium.find_element_by_id("clockbox0").is_displayed())
+        self.assertTrue(self.selenium.find_element(By.ID, "clockbox0").is_displayed())
         self.assertEqual(
             [
                 x.text
-                for x in self.selenium.find_elements_by_xpath(
-                    "//ul[@class='timelist']/li/a"
+                for x in self.selenium.find_elements(
+                    By.XPATH, "//ul[@class='timelist']/li/a"
                 )
             ],
             ["Now", "Midnight", "6 a.m.", "Noon", "6 p.m."],
         )
         # Press the ESC key
-        self.selenium.find_element_by_tag_name("body").send_keys([Keys.ESCAPE])
+        self.selenium.find_element(By.TAG_NAME, "body").send_keys([Keys.ESCAPE])
         # The time picker is hidden again
-        self.assertFalse(self.selenium.find_element_by_id("clockbox0").is_displayed())
+        self.assertFalse(self.selenium.find_element(By.ID, "clockbox0").is_displayed())
         # Click the time icon, then select the 'Noon' value
         time_icon.click()
-        self.selenium.find_element_by_xpath("//a[contains(text(), 'Noon')]").click()
-        self.assertFalse(self.selenium.find_element_by_id("clockbox0").is_displayed())
+        self.selenium.find_element(By.XPATH, "//a[contains(text(), 'Noon')]").click()
+        self.assertFalse(self.selenium.find_element(By.ID, "clockbox0").is_displayed())
         self.assertEqual(
-            self.selenium.find_element_by_id("id_birthdate_1").get_attribute("value"),
+            self.selenium.find_element(By.ID, "id_birthdate_1").get_attribute("value"),
             "12:00:00",
         )
 
@@ -950,6 +974,8 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
         Ensure cells that are not days of the month have the `nonday` CSS class.
         Refs #4574.
         """
+        from selenium.webdriver.common.by import By
+
         self.admin_login(username="super", password="secret", login_url="/")
         # Open a page that has a date and time picker widgets
         self.selenium.get(
@@ -957,14 +983,14 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
         )
 
         # fill in the birth date.
-        self.selenium.find_element_by_id("id_birthdate_0").send_keys("2013-06-01")
+        self.selenium.find_element(By.ID, "id_birthdate_0").send_keys("2013-06-01")
 
         # Click the calendar icon
-        self.selenium.find_element_by_id("calendarlink0").click()
+        self.selenium.find_element(By.ID, "calendarlink0").click()
 
         # get all the tds within the calendar
-        calendar0 = self.selenium.find_element_by_id("calendarin0")
-        tds = calendar0.find_elements_by_tag_name("td")
+        calendar0 = self.selenium.find_element(By.ID, "calendarin0")
+        tds = calendar0.find_elements(By.TAG_NAME, "td")
 
         # make sure the first and last 6 cells have class nonday
         for td in tds[:6] + tds[-6:]:
@@ -975,6 +1001,8 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
         Ensure cell for the day in the input has the `selected` CSS class.
         Refs #4574.
         """
+        from selenium.webdriver.common.by import By
+
         self.admin_login(username="super", password="secret", login_url="/")
         # Open a page that has a date and time picker widgets
         self.selenium.get(
@@ -982,14 +1010,14 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
         )
 
         # fill in the birth date.
-        self.selenium.find_element_by_id("id_birthdate_0").send_keys("2013-06-01")
+        self.selenium.find_element(By.ID, "id_birthdate_0").send_keys("2013-06-01")
 
         # Click the calendar icon
-        self.selenium.find_element_by_id("calendarlink0").click()
+        self.selenium.find_element(By.ID, "calendarlink0").click()
 
         # get all the tds within the calendar
-        calendar0 = self.selenium.find_element_by_id("calendarin0")
-        tds = calendar0.find_elements_by_tag_name("td")
+        calendar0 = self.selenium.find_element(By.ID, "calendarin0")
+        tds = calendar0.find_elements(By.TAG_NAME, "td")
 
         # verify the selected cell
         selected = tds[6]
@@ -1002,6 +1030,8 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
         Ensure no cells are given the selected class when the field is empty.
         Refs #4574.
         """
+        from selenium.webdriver.common.by import By
+
         self.admin_login(username="super", password="secret", login_url="/")
         # Open a page that has a date and time picker widgets
         self.selenium.get(
@@ -1009,11 +1039,11 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
         )
 
         # Click the calendar icon
-        self.selenium.find_element_by_id("calendarlink0").click()
+        self.selenium.find_element(By.ID, "calendarlink0").click()
 
         # get all the tds within the calendar
-        calendar0 = self.selenium.find_element_by_id("calendarin0")
-        tds = calendar0.find_elements_by_tag_name("td")
+        calendar0 = self.selenium.find_element(By.ID, "calendarin0")
+        tds = calendar0.find_elements(By.TAG_NAME, "td")
 
         # verify there are no cells with the selected class
         selected = [td for td in tds if td.get_attribute("class") == "selected"]
@@ -1025,6 +1055,8 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
         The calendar shows the date from the input field for every locale
         supported by Django.
         """
+        from selenium.webdriver.common.by import By
+
         self.selenium.set_window_size(1024, 768)
         self.admin_login(username="super", password="secret", login_url="/")
 
@@ -1059,7 +1091,7 @@ class DateTimePickerSeleniumTests(AdminWidgetSeleniumTestCase):
                 url = reverse("admin:admin_widgets_member_change", args=(member.pk,))
                 self.selenium.get(self.live_server_url + url)
                 # Click on the calendar icon
-                self.selenium.find_element_by_id("calendarlink0").click()
+                self.selenium.find_element(By.ID, "calendarlink0").click()
                 # Make sure that the right month and year are displayed
                 self.wait_for_text("#calendarin0 caption", expected_caption)
 
@@ -1075,6 +1107,8 @@ class DateTimePickerShortcutsSeleniumTests(AdminWidgetSeleniumTestCase):
         in the default time zone "America/Chicago" despite `override_settings` changing
         the time zone to "Asia/Singapore".
         """
+        from selenium.webdriver.common.by import By
+
         self.admin_login(username="super", password="secret", login_url="/")
 
         error_margin = timedelta(seconds=10)
@@ -1091,26 +1125,26 @@ class DateTimePickerShortcutsSeleniumTests(AdminWidgetSeleniumTestCase):
             self.live_server_url + reverse("admin:admin_widgets_member_add")
         )
 
-        self.selenium.find_element_by_id("id_name").send_keys("test")
+        self.selenium.find_element(By.ID, "id_name").send_keys("test")
 
         # Click on the "today" and "now" shortcuts.
-        shortcuts = self.selenium.find_elements_by_css_selector(
-            ".field-birthdate .datetimeshortcuts"
+        shortcuts = self.selenium.find_elements(
+            By.CSS_SELECTOR, ".field-birthdate .datetimeshortcuts"
         )
 
         now = datetime.now()
         for shortcut in shortcuts:
-            shortcut.find_element_by_tag_name("a").click()
+            shortcut.find_element(By.TAG_NAME, "a").click()
 
         # There is a time zone mismatch warning.
         # Warning: This would effectively fail if the TIME_ZONE defined in the
         # settings has the same UTC offset as "Asia/Singapore" because the
         # mismatch warning would be rightfully missing from the page.
-        self.selenium.find_elements_by_css_selector(".field-birthdate .timezonewarning")
+        self.assertCountSeleniumElements(".field-birthdate .timezonewarning", 1)
 
         # Submit the form.
         with self.wait_page_loaded():
-            self.selenium.find_element_by_name("_save").click()
+            self.selenium.find_element(By.NAME, "_save").click()
 
         # Make sure that "now" in JavaScript is within 10 seconds
         # from "now" on the server side.
@@ -1153,6 +1187,8 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
             self.assertEqual(self.has_css_class(remove_all_link, "active"), remove_all)
 
     def execute_basic_operations(self, mode, field_name):
+        from selenium.webdriver.common.by import By
+
         original_url = self.selenium.current_url
 
         from_box = "#id_%s_from" % field_name
@@ -1179,15 +1215,15 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
 
         # Click 'Choose all' --------------------------------------------------
         if mode == "horizontal":
-            self.selenium.find_element_by_id(choose_all_link).click()
+            self.selenium.find_element(By.ID, choose_all_link).click()
         elif mode == "vertical":
             # There 's no 'Choose all' button in vertical mode, so individually
             # select all options and click 'Choose'.
-            for option in self.selenium.find_elements_by_css_selector(
-                from_box + " > option"
+            for option in self.selenium.find_elements(
+                By.CSS_SELECTOR, from_box + " > option"
             ):
                 option.click()
-            self.selenium.find_element_by_id(choose_link).click()
+            self.selenium.find_element(By.ID, choose_link).click()
         self.assertSelectOptions(from_box, [])
         self.assertSelectOptions(
             to_box,
@@ -1206,15 +1242,15 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
 
         # Click 'Remove all' --------------------------------------------------
         if mode == "horizontal":
-            self.selenium.find_element_by_id(remove_all_link).click()
+            self.selenium.find_element(By.ID, remove_all_link).click()
         elif mode == "vertical":
             # There 's no 'Remove all' button in vertical mode, so individually
             # select all options and click 'Remove'.
-            for option in self.selenium.find_elements_by_css_selector(
-                to_box + " > option"
+            for option in self.selenium.find_elements(
+                By.CSS_SELECTOR, to_box + " > option"
             ):
                 option.click()
-            self.selenium.find_element_by_id(remove_link).click()
+            self.selenium.find_element(By.ID, remove_link).click()
         self.assertSelectOptions(
             from_box,
             [
@@ -1232,8 +1268,8 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         self.assertActiveButtons(mode, field_name, False, False, True, False)
 
         # Choose some options ------------------------------------------------
-        from_lisa_select_option = self.selenium.find_element_by_css_selector(
-            '{} > option[value="{}"]'.format(from_box, self.lisa.id)
+        from_lisa_select_option = self.selenium.find_element(
+            By.CSS_SELECTOR, '{} > option[value="{}"]'.format(from_box, self.lisa.id)
         )
 
         # Check the title attribute is there for tool tips: ticket #20821
@@ -1247,7 +1283,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         self.select_option(from_box, str(self.bob.id))
         self.select_option(from_box, str(self.john.id))
         self.assertActiveButtons(mode, field_name, True, False, True, False)
-        self.selenium.find_element_by_id(choose_link).click()
+        self.selenium.find_element(By.ID, choose_link).click()
         self.assertActiveButtons(mode, field_name, False, False, True, True)
 
         self.assertSelectOptions(
@@ -1270,8 +1306,8 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         )
 
         # Check the tooltip is still there after moving: ticket #20821
-        to_lisa_select_option = self.selenium.find_element_by_css_selector(
-            '{} > option[value="{}"]'.format(to_box, self.lisa.id)
+        to_lisa_select_option = self.selenium.find_element(
+            By.CSS_SELECTOR, '{} > option[value="{}"]'.format(to_box, self.lisa.id)
         )
         self.assertEqual(
             to_lisa_select_option.get_attribute("title"),
@@ -1282,7 +1318,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         self.select_option(to_box, str(self.lisa.id))
         self.select_option(to_box, str(self.bob.id))
         self.assertActiveButtons(mode, field_name, False, True, True, True)
-        self.selenium.find_element_by_id(remove_link).click()
+        self.selenium.find_element(By.ID, remove_link).click()
         self.assertActiveButtons(mode, field_name, False, False, True, True)
 
         self.assertSelectOptions(
@@ -1301,7 +1337,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         # Choose some more options --------------------------------------------
         self.select_option(from_box, str(self.arthur.id))
         self.select_option(from_box, str(self.cliff.id))
-        self.selenium.find_element_by_id(choose_link).click()
+        self.selenium.find_element(By.ID, choose_link).click()
 
         self.assertSelectOptions(
             from_box,
@@ -1328,7 +1364,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
 
         # Confirm they're selected after clicking inactive buttons: ticket #26575
         self.assertSelectedOptions(from_box, [str(self.peter.id), str(self.lisa.id)])
-        self.selenium.find_element_by_id(remove_link).click()
+        self.selenium.find_element(By.ID, remove_link).click()
         self.assertSelectedOptions(from_box, [str(self.peter.id), str(self.lisa.id)])
 
         # Unselect the options ------------------------------------------------
@@ -1341,7 +1377,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
 
         # Confirm they're selected after clicking inactive buttons: ticket #26575
         self.assertSelectedOptions(to_box, [str(self.jason.id), str(self.john.id)])
-        self.selenium.find_element_by_id(choose_link).click()
+        self.selenium.find_element(By.ID, choose_link).click()
         self.assertSelectedOptions(to_box, [str(self.jason.id), str(self.john.id)])
 
         # Unselect the options ------------------------------------------------
@@ -1352,6 +1388,8 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         self.assertEqual(self.selenium.current_url, original_url)
 
     def test_basic(self):
+        from selenium.webdriver.common.by import By
+
         self.selenium.set_window_size(1024, 768)
         self.school.students.set([self.lisa, self.peter])
         self.school.alumni.set([self.lisa, self.peter])
@@ -1367,7 +1405,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         self.execute_basic_operations("horizontal", "alumni")
 
         # Save and check that everything is properly stored in the database ---
-        self.selenium.find_element_by_xpath('//input[@value="Save"]').click()
+        self.selenium.find_element(By.XPATH, '//input[@value="Save"]').click()
         self.wait_page_ready()
         self.school = School.objects.get(id=self.school.id)  # Reload from database
         self.assertEqual(
@@ -1384,6 +1422,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         Typing in the search box filters out options displayed in the 'from'
         box.
         """
+        from selenium.webdriver.common.by import By
         from selenium.webdriver.common.keys import Keys
 
         self.selenium.set_window_size(1024, 768)
@@ -1401,7 +1440,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
             to_box = "#id_%s_to" % field_name
             choose_link = "id_%s_add_link" % field_name
             remove_link = "id_%s_remove_link" % field_name
-            input = self.selenium.find_element_by_id("id_%s_input" % field_name)
+            input = self.selenium.find_element(By.ID, "id_%s_input" % field_name)
 
             # Initial values
             self.assertSelectOptions(
@@ -1449,7 +1488,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
                 from_box, [str(self.arthur.id), str(self.jason.id)]
             )
             self.select_option(from_box, str(self.jason.id))
-            self.selenium.find_element_by_id(choose_link).click()
+            self.selenium.find_element(By.ID, choose_link).click()
             self.assertSelectOptions(from_box, [str(self.arthur.id)])
             self.assertSelectOptions(
                 to_box,
@@ -1461,7 +1500,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
             )
 
             self.select_option(to_box, str(self.lisa.id))
-            self.selenium.find_element_by_id(remove_link).click()
+            self.selenium.find_element(By.ID, remove_link).click()
             self.assertSelectOptions(from_box, [str(self.arthur.id), str(self.lisa.id)])
             self.assertSelectOptions(to_box, [str(self.peter.id), str(self.jason.id)])
 
@@ -1483,7 +1522,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
             # Pressing enter on a filtered option sends it properly to
             # the 'to' box.
             self.select_option(to_box, str(self.jason.id))
-            self.selenium.find_element_by_id(remove_link).click()
+            self.selenium.find_element(By.ID, remove_link).click()
             input.send_keys("ja")
             self.assertSelectOptions(from_box, [str(self.jason.id)])
             input.send_keys([Keys.ENTER])
@@ -1492,7 +1531,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
 
         # Save and check that everything is properly stored in the database ---
         with self.wait_page_loaded():
-            self.selenium.find_element_by_xpath('//input[@value="Save"]').click()
+            self.selenium.find_element(By.XPATH, '//input[@value="Save"]').click()
         self.school = School.objects.get(id=self.school.id)  # Reload from database
         self.assertEqual(list(self.school.students.all()), [self.jason, self.peter])
         self.assertEqual(list(self.school.alumni.all()), [self.jason, self.peter])
@@ -1503,6 +1542,8 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         and then clicking the browser's back button would clear the
         filter_horizontal/filter_vertical widgets (#13614).
         """
+        from selenium.webdriver.common.by import By
+
         self.school.students.set([self.lisa, self.peter])
         self.school.alumni.set([self.lisa, self.peter])
         self.admin_login(username="super", password="secret", login_url="/")
@@ -1511,7 +1552,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         )
         self.selenium.get(self.live_server_url + change_url)
         # Navigate away and go back to the change form page.
-        self.selenium.find_element_by_link_text("Home").click()
+        self.selenium.find_element(By.LINK_TEXT, "Home").click()
         self.selenium.back()
         expected_unselected_values = [
             str(self.arthur.id),
@@ -1542,10 +1583,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         )
         self.selenium.get(self.live_server_url + change_url)
 
-        options_len = len(
-            self.selenium.find_elements_by_css_selector("#id_students_to > option")
-        )
-        self.assertEqual(options_len, 2)
+        self.assertCountSeleniumElements("#id_students_to > option", 2)
 
         # self.selenium.refresh() or send_keys(Keys.F5) does hard reload and
         # doesn't replicate what happens when a user clicks the browser's
@@ -1553,10 +1591,7 @@ class HorizontalVerticalFilterSeleniumTests(AdminWidgetSeleniumTestCase):
         with self.wait_page_loaded():
             self.selenium.execute_script("location.reload()")
 
-        options_len = len(
-            self.selenium.find_elements_by_css_selector("#id_students_to > option")
-        )
-        self.assertEqual(options_len, 2)
+        self.assertCountSeleniumElements("#id_students_to > option", 2)
 
 
 class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
@@ -1566,6 +1601,8 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
         Band.objects.create(id=98, name="Green Potatoes")
 
     def test_ForeignKey(self):
+        from selenium.webdriver.common.by import By
+
         self.admin_login(username="super", password="secret", login_url="/")
         self.selenium.get(
             self.live_server_url + reverse("admin:admin_widgets_event_add")
@@ -1574,13 +1611,13 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
 
         # No value has been selected yet
         self.assertEqual(
-            self.selenium.find_element_by_id("id_main_band").get_attribute("value"), ""
+            self.selenium.find_element(By.ID, "id_main_band").get_attribute("value"), ""
         )
 
         # Open the popup window and click on a band
-        self.selenium.find_element_by_id("lookup_id_main_band").click()
+        self.selenium.find_element(By.ID, "lookup_id_main_band").click()
         self.wait_for_and_switch_to_popup()
-        link = self.selenium.find_element_by_link_text("Bogey Blues")
+        link = self.selenium.find_element(By.LINK_TEXT, "Bogey Blues")
         self.assertIn("/band/42/", link.get_attribute("href"))
         link.click()
 
@@ -1589,9 +1626,9 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
         self.wait_for_value("#id_main_band", "42")
 
         # Reopen the popup window and click on another band
-        self.selenium.find_element_by_id("lookup_id_main_band").click()
+        self.selenium.find_element(By.ID, "lookup_id_main_band").click()
         self.wait_for_and_switch_to_popup()
-        link = self.selenium.find_element_by_link_text("Green Potatoes")
+        link = self.selenium.find_element(By.LINK_TEXT, "Green Potatoes")
         self.assertIn("/band/98/", link.get_attribute("href"))
         link.click()
 
@@ -1600,6 +1637,8 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
         self.wait_for_value("#id_main_band", "98")
 
     def test_many_to_many(self):
+        from selenium.webdriver.common.by import By
+
         self.admin_login(username="super", password="secret", login_url="/")
         self.selenium.get(
             self.live_server_url + reverse("admin:admin_widgets_event_add")
@@ -1608,7 +1647,7 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
 
         # No value has been selected yet
         self.assertEqual(
-            self.selenium.find_element_by_id("id_supporting_bands").get_attribute(
+            self.selenium.find_element(By.ID, "id_supporting_bands").get_attribute(
                 "value"
             ),
             "",
@@ -1616,16 +1655,16 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
 
         # Help text for the field is displayed
         self.assertEqual(
-            self.selenium.find_element_by_css_selector(
-                ".field-supporting_bands div.help"
+            self.selenium.find_element(
+                By.CSS_SELECTOR, ".field-supporting_bands div.help"
             ).text,
             "Supporting Bands.",
         )
 
         # Open the popup window and click on a band
-        self.selenium.find_element_by_id("lookup_id_supporting_bands").click()
+        self.selenium.find_element(By.ID, "lookup_id_supporting_bands").click()
         self.wait_for_and_switch_to_popup()
-        link = self.selenium.find_element_by_link_text("Bogey Blues")
+        link = self.selenium.find_element(By.LINK_TEXT, "Bogey Blues")
         self.assertIn("/band/42/", link.get_attribute("href"))
         link.click()
 
@@ -1634,9 +1673,9 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
         self.wait_for_value("#id_supporting_bands", "42")
 
         # Reopen the popup window and click on another band
-        self.selenium.find_element_by_id("lookup_id_supporting_bands").click()
+        self.selenium.find_element(By.ID, "lookup_id_supporting_bands").click()
         self.wait_for_and_switch_to_popup()
-        link = self.selenium.find_element_by_link_text("Green Potatoes")
+        link = self.selenium.find_element(By.LINK_TEXT, "Green Potatoes")
         self.assertIn("/band/98/", link.get_attribute("href"))
         link.click()
 
@@ -1647,6 +1686,9 @@ class AdminRawIdWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
 
 class RelatedFieldWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
     def test_ForeignKey_using_to_field(self):
+        from selenium.webdriver.common.by import By
+        from selenium.webdriver.support.ui import Select
+
         self.admin_login(username="super", password="secret", login_url="/")
         self.selenium.get(
             self.live_server_url + reverse("admin:admin_widgets_profile_add")
@@ -1654,39 +1696,51 @@ class RelatedFieldWidgetSeleniumTests(AdminWidgetSeleniumTestCase):
 
         main_window = self.selenium.current_window_handle
         # Click the Add User button to add new
-        self.selenium.find_element_by_id("add_id_user").click()
+        self.selenium.find_element(By.ID, "add_id_user").click()
         self.wait_for_and_switch_to_popup()
-        password_field = self.selenium.find_element_by_id("id_password")
+        password_field = self.selenium.find_element(By.ID, "id_password")
         password_field.send_keys("password")
 
-        username_field = self.selenium.find_element_by_id("id_username")
+        username_field = self.selenium.find_element(By.ID, "id_username")
         username_value = "newuser"
         username_field.send_keys(username_value)
 
         save_button_css_selector = ".submit-row > input[type=submit]"
-        self.selenium.find_element_by_css_selector(save_button_css_selector).click()
+        self.selenium.find_element(By.CSS_SELECTOR, save_button_css_selector).click()
         self.selenium.switch_to.window(main_window)
         # The field now contains the new user
-        self.selenium.find_element_by_css_selector("#id_user option[value=newuser]")
+        self.selenium.find_element(By.CSS_SELECTOR, "#id_user option[value=newuser]")
 
+        self.selenium.find_element(By.ID, "view_id_user").click()
+        self.wait_for_value("#id_username", "newuser")
+        self.selenium.back()
+
+        select = Select(self.selenium.find_element(By.ID, "id_user"))
+        select.select_by_value("newuser")
         # Click the Change User button to change it
-        self.selenium.find_element_by_id("change_id_user").click()
+        self.selenium.find_element(By.ID, "change_id_user").click()
         self.wait_for_and_switch_to_popup()
 
-        username_field = self.selenium.find_element_by_id("id_username")
+        username_field = self.selenium.find_element(By.ID, "id_username")
         username_value = "changednewuser"
         username_field.clear()
         username_field.send_keys(username_value)
 
         save_button_css_selector = ".submit-row > input[type=submit]"
-        self.selenium.find_element_by_css_selector(save_button_css_selector).click()
+        self.selenium.find_element(By.CSS_SELECTOR, save_button_css_selector).click()
         self.selenium.switch_to.window(main_window)
-        self.selenium.find_element_by_css_selector(
-            "#id_user option[value=changednewuser]"
+        self.selenium.find_element(
+            By.CSS_SELECTOR, "#id_user option[value=changednewuser]"
         )
 
+        self.selenium.find_element(By.ID, "view_id_user").click()
+        self.wait_for_value("#id_username", "changednewuser")
+        self.selenium.back()
+
+        select = Select(self.selenium.find_element(By.ID, "id_user"))
+        select.select_by_value("changednewuser")
         # Go ahead and submit the form to make sure it works
-        self.selenium.find_element_by_css_selector(save_button_css_selector).click()
+        self.selenium.find_element(By.CSS_SELECTOR, save_button_css_selector).click()
         self.wait_for_text(
             "li.success", "The profile “changednewuser” was added successfully."
         )
