@@ -178,7 +178,7 @@ class AggregateTestCase(TestCase):
         s3.books.add(cls.b3, cls.b4, cls.b6)
 
     def test_empty_aggregate(self):
-        self.assertEqual(Author.objects.all().aggregate(), {})
+        self.assertEqual(Author.objects.aggregate(), {})
 
     def test_aggregate_in_order_by(self):
         msg = (
@@ -209,11 +209,7 @@ class AggregateTestCase(TestCase):
         vals = Book.objects.filter(rating__lt=4.5).aggregate(Avg("authors__age"))
         self.assertEqual(vals, {"authors__age__avg": Approximate(38.2857, places=2)})
 
-        vals = (
-            Author.objects.all()
-            .filter(name__contains="a")
-            .aggregate(Avg("book__rating"))
-        )
+        vals = Author.objects.filter(name__contains="a").aggregate(Avg("book__rating"))
         self.assertEqual(vals, {"book__rating__avg": 4.0})
 
         vals = Book.objects.aggregate(Sum("publisher__num_awards"))
@@ -1016,7 +1012,7 @@ class AggregateTestCase(TestCase):
         """
         Aggregation over sliced queryset works correctly.
         """
-        qs = Book.objects.all().order_by("-rating")[0:3]
+        qs = Book.objects.order_by("-rating")[0:3]
         vals = qs.aggregate(average_top3_rating=Avg("rating"))["average_top3_rating"]
         self.assertAlmostEqual(vals, 4.5, places=2)
 
@@ -1026,8 +1022,7 @@ class AggregateTestCase(TestCase):
         select_related() stuff.
         """
         qs = (
-            Book.objects.all()
-            .select_for_update()
+            Book.objects.select_for_update()
             .order_by("pk")
             .select_related("publisher")
             .annotate(max_pk=Max("pk"))
@@ -1131,8 +1126,8 @@ class AggregateTestCase(TestCase):
 
     def test_combine_different_types(self):
         msg = (
-            "Expression contains mixed types: FloatField, DecimalField. "
-            "You must set output_field."
+            "Cannot infer type of '+' expression involving these types: FloatField, "
+            "DecimalField. You must set output_field."
         )
         qs = Book.objects.annotate(sums=Sum("rating") + Sum("pages") + Sum("price"))
         with self.assertRaisesMessage(FieldError, msg):
@@ -1439,6 +1434,18 @@ class AggregateTestCase(TestCase):
         )
         self.assertTrue(publisher_qs.exists())
 
+    def test_aggregation_filter_exists(self):
+        publishers_having_more_than_one_book_qs = (
+            Book.objects.values("publisher")
+            .annotate(cnt=Count("isbn"))
+            .filter(cnt__gt=1)
+        )
+        query = publishers_having_more_than_one_book_qs.query.exists(
+            using=connection.alias
+        )
+        _, _, group_by = query.get_compiler(connection=connection).pre_sql_setup()
+        self.assertEqual(len(group_by), 1)
+
     def test_aggregation_exists_annotation(self):
         published_books = Book.objects.filter(publisher=OuterRef("pk"))
         publisher_qs = Publisher.objects.annotate(
@@ -1579,10 +1586,17 @@ class AggregateTestCase(TestCase):
             .annotate(count=Count("pk"))
             .values("count")
         )
-        long_books_count_breakdown = Publisher.objects.values_list(
-            Subquery(long_books_count_qs, IntegerField()),
-        ).annotate(total=Count("*"))
-        self.assertEqual(dict(long_books_count_breakdown), {None: 1, 1: 4})
+        groups = [
+            Subquery(long_books_count_qs),
+            long_books_count_qs,
+            long_books_count_qs.query,
+        ]
+        for group in groups:
+            with self.subTest(group=group.__class__.__name__):
+                long_books_count_breakdown = Publisher.objects.values_list(
+                    group,
+                ).annotate(total=Count("*"))
+                self.assertEqual(dict(long_books_count_breakdown), {None: 1, 1: 4})
 
     @skipUnlessDBFeature("supports_subqueries_in_group_by")
     def test_group_by_exists_annotation(self):
@@ -1649,6 +1663,23 @@ class AggregateTestCase(TestCase):
         ).values_list("publisher_count", flat=True)
         self.assertSequenceEqual(books_breakdown, [1] * 6)
 
+    def test_filter_in_subquery_or_aggregation(self):
+        """
+        Filtering against an aggregate requires the usage of the HAVING clause.
+
+        If such a filter is unionized to a non-aggregate one the latter will
+        also need to be moved to the HAVING clause and have its grouping
+        columns used in the GROUP BY.
+
+        When this is done with a subquery the specialized logic in charge of
+        using outer reference columns to group should be used instead of the
+        subquery itself as the latter might return multiple rows.
+        """
+        authors = Author.objects.annotate(
+            Count("book"),
+        ).filter(Q(book__count__gt=0) | Q(pk__in=Book.objects.values("authors")))
+        self.assertQuerysetEqual(authors, Author.objects.all(), ordered=False)
+
     def test_aggregation_random_ordering(self):
         """Random() is not included in the GROUP BY when used for ordering."""
         authors = Author.objects.annotate(contact_count=Count("book")).order_by("?")
@@ -1828,7 +1859,7 @@ class AggregateTestCase(TestCase):
         )
 
     def test_aggregation_default_using_time_from_database(self):
-        now = timezone.now().astimezone(timezone.utc)
+        now = timezone.now().astimezone(datetime.timezone.utc)
         expr = Min(
             "store__friday_night_closing",
             filter=~Q(store__name="Amazon.com"),
@@ -1880,7 +1911,7 @@ class AggregateTestCase(TestCase):
         )
 
     def test_aggregation_default_using_date_from_database(self):
-        now = timezone.now().astimezone(timezone.utc)
+        now = timezone.now().astimezone(datetime.timezone.utc)
         expr = Min("book__pubdate", default=TruncDate(NowUTC()))
         queryset = Publisher.objects.annotate(earliest_pubdate=expr).order_by("name")
         self.assertSequenceEqual(
@@ -1941,7 +1972,7 @@ class AggregateTestCase(TestCase):
         )
 
     def test_aggregation_default_using_datetime_from_database(self):
-        now = timezone.now().astimezone(timezone.utc)
+        now = timezone.now().astimezone(datetime.timezone.utc)
         expr = Min(
             "store__original_opening",
             filter=~Q(store__name="Amazon.com"),
@@ -2023,7 +2054,7 @@ class AggregateTestCase(TestCase):
         self.assertEqual(result["num_awards__sum"], 20)
 
     def test_exists_none_with_aggregate(self):
-        qs = Book.objects.all().annotate(
+        qs = Book.objects.annotate(
             count=Count("id"),
             exists=Exists(Author.objects.none()),
         )
@@ -2039,7 +2070,7 @@ class AggregateTestCase(TestCase):
             Author.objects.aggregate(**{crafted_alias: Avg("age")})
 
     def test_exists_extra_where_with_aggregate(self):
-        qs = Book.objects.all().annotate(
+        qs = Book.objects.annotate(
             count=Count("id"),
             exists=Exists(Author.objects.extra(where=["1=0"])),
         )
