@@ -13,6 +13,7 @@ except ImportError:
     pytz = None
 
 from django.conf import settings
+from django.db import DataError, OperationalError
 from django.db.models import (
     DateField,
     DateTimeField,
@@ -244,8 +245,7 @@ class DateFunctionTests(TestCase):
         self.create_model(start_datetime, end_datetime)
         self.create_model(end_datetime, start_datetime)
 
-        msg = "Invalid lookup_name: "
-        with self.assertRaisesMessage(ValueError, msg):
+        with self.assertRaises((DataError, OperationalError, ValueError)):
             DTModel.objects.filter(
                 start_datetime__year=Extract(
                     "start_datetime", "day' FROM start_datetime)) OR 1=1;--"
@@ -916,6 +916,22 @@ class DateFunctionTests(TestCase):
             2,
         )
 
+    def test_extract_second_func_no_fractional(self):
+        start_datetime = datetime(2015, 6, 15, 14, 30, 50, 321)
+        end_datetime = datetime(2016, 6, 15, 14, 30, 50, 783)
+        if settings.USE_TZ:
+            start_datetime = timezone.make_aware(start_datetime)
+            end_datetime = timezone.make_aware(end_datetime)
+        obj = self.create_model(start_datetime, end_datetime)
+        self.assertSequenceEqual(
+            DTModel.objects.filter(start_datetime__second=F("end_datetime__second")),
+            [obj],
+        )
+        self.assertSequenceEqual(
+            DTModel.objects.filter(start_time__second=F("end_time__second")),
+            [obj],
+        )
+
     def test_trunc_lookup_name_sql_injection(self):
         start_datetime = datetime(2015, 6, 15, 14, 30, 50, 321)
         end_datetime = datetime(2016, 6, 15, 14, 10, 50, 123)
@@ -924,17 +940,21 @@ class DateFunctionTests(TestCase):
             end_datetime = timezone.make_aware(end_datetime)
         self.create_model(start_datetime, end_datetime)
         self.create_model(end_datetime, start_datetime)
-        msg = "Invalid kind: "
-        with self.assertRaisesMessage(ValueError, msg):
-            DTModel.objects.filter(
+        # Database backends raise an exception or don't return any results.
+        try:
+            exists = DTModel.objects.filter(
                 start_datetime__date=Trunc(
                     "start_datetime",
                     "year', start_datetime)) OR 1=1;--",
                 )
             ).exists()
+        except (DataError, OperationalError):
+            pass
+        else:
+            self.assertIs(exists, False)
 
     def test_trunc_func(self):
-        start_datetime = datetime(2015, 6, 15, 14, 30, 50, 321)
+        start_datetime = datetime(999, 6, 15, 14, 30, 50, 321)
         end_datetime = datetime(2016, 6, 15, 14, 10, 50, 123)
         if settings.USE_TZ:
             start_datetime = timezone.make_aware(start_datetime)
@@ -942,54 +962,6 @@ class DateFunctionTests(TestCase):
         self.create_model(start_datetime, end_datetime)
         self.create_model(end_datetime, start_datetime)
 
-        msg = "output_field must be either DateField, TimeField, or DateTimeField"
-        with self.assertRaisesMessage(ValueError, msg):
-            list(
-                DTModel.objects.annotate(
-                    truncated=Trunc(
-                        "start_datetime", "year", output_field=IntegerField()
-                    )
-                )
-            )
-
-        msg = "'name' isn't a DateField, TimeField, or DateTimeField."
-        with self.assertRaisesMessage(TypeError, msg):
-            list(
-                DTModel.objects.annotate(
-                    truncated=Trunc("name", "year", output_field=DateTimeField())
-                )
-            )
-
-        with self.assertRaisesMessage(
-            ValueError, "Cannot truncate DateField 'start_date' to DateTimeField"
-        ):
-            list(DTModel.objects.annotate(truncated=Trunc("start_date", "second")))
-
-        with self.assertRaisesMessage(
-            ValueError, "Cannot truncate TimeField 'start_time' to DateTimeField"
-        ):
-            list(DTModel.objects.annotate(truncated=Trunc("start_time", "month")))
-
-        with self.assertRaisesMessage(
-            ValueError, "Cannot truncate DateField 'start_date' to DateTimeField"
-        ):
-            list(
-                DTModel.objects.annotate(
-                    truncated=Trunc("start_date", "month", output_field=DateTimeField())
-                )
-            )
-
-        with self.assertRaisesMessage(
-            ValueError, "Cannot truncate TimeField 'start_time' to DateTimeField"
-        ):
-            list(
-                DTModel.objects.annotate(
-                    truncated=Trunc(
-                        "start_time", "second", output_field=DateTimeField()
-                    )
-                )
-            )
-
         def test_datetime_kind(kind):
             self.assertQuerysetEqual(
                 DTModel.objects.annotate(
@@ -1043,7 +1015,6 @@ class DateFunctionTests(TestCase):
         test_date_kind("year")
         test_date_kind("quarter")
         test_date_kind("month")
-        test_date_kind("week")
         test_date_kind("day")
         test_time_kind("hour")
         test_time_kind("minute")
@@ -1051,7 +1022,6 @@ class DateFunctionTests(TestCase):
         test_datetime_kind("year")
         test_datetime_kind("quarter")
         test_datetime_kind("month")
-        test_datetime_kind("week")
         test_datetime_kind("day")
         test_datetime_kind("hour")
         test_datetime_kind("minute")
@@ -1067,6 +1037,86 @@ class DateFunctionTests(TestCase):
         )
         self.assertEqual(qs.count(), 2)
 
+    def _test_trunc_week(self, start_datetime, end_datetime):
+        if settings.USE_TZ:
+            start_datetime = timezone.make_aware(start_datetime)
+            end_datetime = timezone.make_aware(end_datetime)
+        self.create_model(start_datetime, end_datetime)
+        self.create_model(end_datetime, start_datetime)
+
+        self.assertQuerysetEqual(
+            DTModel.objects.annotate(
+                truncated=Trunc("start_datetime", "week", output_field=DateTimeField())
+            ).order_by("start_datetime"),
+            [
+                (start_datetime, truncate_to(start_datetime, "week")),
+                (end_datetime, truncate_to(end_datetime, "week")),
+            ],
+            lambda m: (m.start_datetime, m.truncated),
+        )
+        self.assertQuerysetEqual(
+            DTModel.objects.annotate(
+                truncated=Trunc("start_date", "week", output_field=DateField())
+            ).order_by("start_datetime"),
+            [
+                (start_datetime, truncate_to(start_datetime.date(), "week")),
+                (end_datetime, truncate_to(end_datetime.date(), "week")),
+            ],
+            lambda m: (m.start_datetime, m.truncated),
+        )
+
+    def test_trunc_week(self):
+        self._test_trunc_week(
+            start_datetime=datetime(2015, 6, 15, 14, 30, 50, 321),
+            end_datetime=datetime(2016, 6, 15, 14, 10, 50, 123),
+        )
+
+    def test_trunc_week_before_1000(self):
+        self._test_trunc_week(
+            start_datetime=datetime(999, 6, 15, 14, 30, 50, 321),
+            end_datetime=datetime(2016, 6, 15, 14, 10, 50, 123),
+        )
+
+    def test_trunc_invalid_arguments(self):
+        msg = "output_field must be either DateField, TimeField, or DateTimeField"
+        with self.assertRaisesMessage(ValueError, msg):
+            list(
+                DTModel.objects.annotate(
+                    truncated=Trunc(
+                        "start_datetime", "year", output_field=IntegerField()
+                    ),
+                )
+            )
+        msg = "'name' isn't a DateField, TimeField, or DateTimeField."
+        with self.assertRaisesMessage(TypeError, msg):
+            list(
+                DTModel.objects.annotate(
+                    truncated=Trunc("name", "year", output_field=DateTimeField()),
+                )
+            )
+        msg = "Cannot truncate DateField 'start_date' to DateTimeField"
+        with self.assertRaisesMessage(ValueError, msg):
+            list(DTModel.objects.annotate(truncated=Trunc("start_date", "second")))
+        with self.assertRaisesMessage(ValueError, msg):
+            list(
+                DTModel.objects.annotate(
+                    truncated=Trunc(
+                        "start_date", "month", output_field=DateTimeField()
+                    ),
+                )
+            )
+        msg = "Cannot truncate TimeField 'start_time' to DateTimeField"
+        with self.assertRaisesMessage(ValueError, msg):
+            list(DTModel.objects.annotate(truncated=Trunc("start_time", "month")))
+        with self.assertRaisesMessage(ValueError, msg):
+            list(
+                DTModel.objects.annotate(
+                    truncated=Trunc(
+                        "start_time", "second", output_field=DateTimeField()
+                    ),
+                )
+            )
+
     def test_trunc_none(self):
         self.create_model(None, None)
         for t in (
@@ -1594,7 +1644,7 @@ class DateFunctionTests(TestCase):
         outer = Author.objects.annotate(
             newest_fan_year=TruncYear(Subquery(inner, output_field=DateTimeField()))
         )
-        tz = timezone.utc if settings.USE_TZ else None
+        tz = datetime_timezone.utc if settings.USE_TZ else None
         self.assertSequenceEqual(
             outer.order_by("name").values("name", "newest_fan_year"),
             [
@@ -1746,7 +1796,7 @@ class DateFunctionWithTimeZoneTests(DateFunctionTests):
                         DTModel.objects.annotate(
                             day_melb=Extract("start_datetime", "day"),
                             day_utc=Extract(
-                                "start_datetime", "day", tzinfo=timezone.utc
+                                "start_datetime", "day", tzinfo=datetime_timezone.utc
                             ),
                         )
                         .order_by("start_datetime")
@@ -1814,9 +1864,8 @@ class DateFunctionWithTimeZoneTests(DateFunctionTests):
     @ignore_warnings(category=RemovedInDjango50Warning)
     def test_trunc_ambiguous_and_invalid_times(self):
         sao = pytz.timezone("America/Sao_Paulo")
-        utc = timezone.utc
-        start_datetime = datetime(2016, 10, 16, 13, tzinfo=utc)
-        end_datetime = datetime(2016, 2, 21, 1, tzinfo=utc)
+        start_datetime = datetime(2016, 10, 16, 13, tzinfo=datetime_timezone.utc)
+        end_datetime = datetime(2016, 2, 21, 1, tzinfo=datetime_timezone.utc)
         self.create_model(start_datetime, end_datetime)
         with timezone.override(sao):
             with self.assertRaisesMessage(
