@@ -379,6 +379,15 @@ class CaseExpressionTests(TestCase):
             transform=attrgetter("integer", "test"),
         )
 
+    def test_annotate_filter_decimal(self):
+        obj = CaseTestModel.objects.create(integer=0, decimal=Decimal("1"))
+        qs = CaseTestModel.objects.annotate(
+            x=Case(When(integer=0, then=F("decimal"))),
+            y=Case(When(integer=0, then=Value(Decimal("1")))),
+        )
+        self.assertSequenceEqual(qs.filter(Q(x=1) & Q(x=Decimal("1"))), [obj])
+        self.assertSequenceEqual(qs.filter(Q(y=1) & Q(y=Decimal("1"))), [obj])
+
     def test_annotate_values_not_in_order_by(self):
         self.assertEqual(
             list(
@@ -712,7 +721,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, "one"),
                 (2, "two"),
@@ -733,7 +742,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, 1), (2, 2), (3, None), (2, 2), (3, None), (3, None), (4, None)],
             transform=attrgetter("integer", "integer2"),
         )
@@ -747,7 +756,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [("1", 2), ("2", 5), ("3", 3), ("2", 5), ("3", 3), ("3", 3), ("4", 4)],
             transform=attrgetter("string", "integer"),
         )
@@ -760,7 +769,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, "equal"),
                 (2, "+1"),
@@ -805,7 +814,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, 1), (2, 2), (3, None), (2, 2), (3, None), (3, None), (4, None)],
             transform=attrgetter("integer", "big_integer"),
         )
@@ -819,7 +828,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, b"one"),
                 (2, b"two"),
@@ -841,7 +850,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, True),
                 (2, True),
@@ -862,7 +871,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, date(2015, 1, 1)),
                 (2, date(2015, 1, 2)),
@@ -883,7 +892,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, datetime(2015, 1, 1)),
                 (2, datetime(2015, 1, 2)),
@@ -906,7 +915,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, Decimal("1.1")),
                 (2, Decimal("2.2")),
@@ -927,7 +936,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, timedelta(1)),
                 (2, timedelta(2)),
@@ -949,7 +958,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, "1@example.com"),
                 (2, "2@example.com"),
@@ -970,7 +979,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, "~/1"), (2, "~/2"), (3, ""), (2, "~/2"), (3, ""), (3, ""), (4, "")],
             transform=lambda o: (o.integer, str(o.file)),
         )
@@ -984,7 +993,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, "~/1"), (2, "~/2"), (3, ""), (2, "~/2"), (3, ""), (3, ""), (4, "")],
             transform=attrgetter("integer", "file_path"),
         )
@@ -997,7 +1006,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, 1.1), (2, 2.2), (3, None), (2, 2.2), (3, None), (3, None), (4, None)],
             transform=attrgetter("integer", "float"),
         )
@@ -1011,7 +1020,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, "~/1"), (2, "~/2"), (3, ""), (2, "~/2"), (3, ""), (3, ""), (4, "")],
             transform=lambda o: (o.integer, str(o.image)),
         )
@@ -1025,7 +1034,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, "1.1.1.1"),
                 (2, "2.2.2.2"),
@@ -1046,7 +1055,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, True),
                 (2, False),
@@ -1067,7 +1076,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, 1), (2, 2), (3, None), (2, 2), (3, None), (3, None), (4, None)],
             transform=attrgetter("integer", "positive_big_integer"),
         )
@@ -1080,7 +1089,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, 1), (2, 2), (3, None), (2, 2), (3, None), (3, None), (4, None)],
             transform=attrgetter("integer", "positive_integer"),
         )
@@ -1093,7 +1102,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, 1), (2, 2), (3, None), (2, 2), (3, None), (3, None), (4, None)],
             transform=attrgetter("integer", "positive_small_integer"),
         )
@@ -1107,7 +1116,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, "1"), (2, "2"), (3, ""), (2, "2"), (3, ""), (3, ""), (4, "")],
             transform=attrgetter("integer", "slug"),
         )
@@ -1120,7 +1129,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, 1), (2, 2), (3, None), (2, 2), (3, None), (3, None), (4, None)],
             transform=attrgetter("integer", "small_integer"),
         )
@@ -1147,7 +1156,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [(1, "1"), (2, "2"), (3, ""), (2, "2"), (3, ""), (3, ""), (4, "")],
             transform=attrgetter("integer", "text"),
         )
@@ -1160,7 +1169,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, time(1)),
                 (2, time(2)),
@@ -1182,7 +1191,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, "http://1.example.com/"),
                 (2, "http://2.example.com/"),
@@ -1203,7 +1212,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, UUID("11111111111111111111111111111111")),
                 (2, UUID("22222222222222222222222222222222")),
@@ -1226,7 +1235,7 @@ class CaseExpressionTests(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            CaseTestModel.objects.all().order_by("pk"),
+            CaseTestModel.objects.order_by("pk"),
             [
                 (1, obj1.pk),
                 (2, obj2.pk),
@@ -1541,7 +1550,7 @@ class CaseDocumentationExamples(TestCase):
             ),
         )
         self.assertQuerysetEqual(
-            Client.objects.all().order_by("pk"),
+            Client.objects.order_by("pk"),
             [("Jane Doe", "G"), ("James Smith", "R"), ("Jack Black", "P")],
             transform=attrgetter("name", "account_type"),
         )
