@@ -364,6 +364,27 @@ class HttpResponseTests(SimpleTestCase):
         with self.assertRaises(BadHeaderError):
             r.headers.__setitem__("test\nstr", "test")
 
+    def test_encoded_with_newlines_in_headers(self):
+        """
+        Keys & values which throw a UnicodeError when encoding/decoding should
+        still be checked for newlines and re-raised as a BadHeaderError.
+        These specifically would still throw BadHeaderError after decoding
+        successfully, because the newlines are sandwiched in the middle of the
+        string and email.Header leaves those as they are.
+        """
+        r = HttpResponse()
+        pairs = (
+            ("†\nother", "test"),
+            ("test", "†\nother"),
+            (b"\xe2\x80\xa0\nother", "test"),
+            ("test", b"\xe2\x80\xa0\nother"),
+        )
+        msg = "Header values can't contain newlines"
+        for key, value in pairs:
+            with self.subTest(key=key, value=value):
+                with self.assertRaisesMessage(BadHeaderError, msg):
+                    r[key] = value
+
     def test_dict_behavior(self):
         """
         Test for bug #14020: Make HttpResponse.get work like dict.get
@@ -518,7 +539,6 @@ class HttpResponseSubclassesTests(SimpleTestCase):
         response = HttpResponseRedirect(
             "/redirected/",
             content="The resource has temporarily moved",
-            content_type="text/html",
         )
         self.assertContains(
             response, "The resource has temporarily moved", status_code=302
@@ -571,7 +591,7 @@ class HttpResponseSubclassesTests(SimpleTestCase):
         self.assertEqual(response.status_code, 405)
         # Standard HttpResponse init args can be used
         response = HttpResponseNotAllowed(
-            ["GET"], content="Only the GET method is allowed", content_type="text/html"
+            ["GET"], content="Only the GET method is allowed"
         )
         self.assertContains(response, "Only the GET method is allowed", status_code=405)
 
