@@ -246,21 +246,11 @@ class ModelInheritanceTest(TestCase):
         self.assertEqual(r.id, orig_id)
         self.assertEqual(r.id, r.place_ptr_id)
 
-    def test_issue_7488(self):
-        # Regression test for #7488. This looks a little crazy, but it's the
-        # equivalent of what the admin interface has to do for the edit-inline
-        # case.
-        suppliers = Supplier.objects.filter(
-            restaurant=Restaurant(name="xx", address="yy")
-        )
-        suppliers = list(suppliers)
-        self.assertEqual(suppliers, [])
-
     def test_issue_11764(self):
         """
         Regression test for #11764
         """
-        wholesalers = list(Wholesaler.objects.all().select_related())
+        wholesalers = list(Wholesaler.objects.select_related())
         self.assertEqual(wholesalers, [])
 
     def test_issue_7853(self):
@@ -426,6 +416,11 @@ class ModelInheritanceTest(TestCase):
         parties = list(p4.bachelorparty_set.all())
         self.assertEqual(parties, [bachelor, messy_parent])
 
+    def test_abstract_base_class_m2m_relation_inheritance_manager_reused(self):
+        p1 = Person.objects.create(name="Alice")
+        self.assertIs(p1.birthdayparty_set, p1.birthdayparty_set)
+        self.assertIs(p1.bachelorparty_set, p1.bachelorparty_set)
+
     def test_abstract_verbose_name_plural_inheritance(self):
         """
         verbose_name_plural correctly inherited from ABC if inheritance chain
@@ -520,7 +515,7 @@ class ModelInheritanceTest(TestCase):
             serves_pizza=True,
             serves_hot_dogs=True,
         )
-        p = Place.objects.all().select_related("restaurant")[0]
+        p = Place.objects.select_related("restaurant")[0]
         self.assertIsInstance(p.restaurant.serves_pizza, bool)
 
     def test_inheritance_select_related(self):
@@ -672,3 +667,15 @@ class ModelInheritanceTest(TestCase):
             Politician.objects.get(pk=c1.politician_ptr_id).title,
             "senator 1",
         )
+
+    def test_mti_update_parent_through_child(self):
+        Politician.objects.create()
+        Congressman.objects.create()
+        Congressman.objects.update(title="senator 1")
+        self.assertEqual(Congressman.objects.get().title, "senator 1")
+
+    def test_mti_update_grand_parent_through_child(self):
+        Politician.objects.create()
+        Senator.objects.create()
+        Senator.objects.update(title="senator 1")
+        self.assertEqual(Senator.objects.get().title, "senator 1")
