@@ -1,3 +1,4 @@
+from django.db import connection
 from django.db.models import (
     CharField,
     F,
@@ -7,9 +8,11 @@ from django.db.models import (
     Q,
     Subquery,
     Value,
+    Window,
 )
 from django.db.models.fields.json import KeyTextTransform, KeyTransform
 from django.db.models.functions import Cast, Concat, Substr
+from django.test import skipUnlessDBFeature
 from django.test.utils import Approximate, ignore_warnings
 from django.utils import timezone
 from django.utils.deprecation import RemovedInDjango50Warning
@@ -22,6 +25,7 @@ try:
         ArrayAgg,
         BitAnd,
         BitOr,
+        BitXor,
         BoolAnd,
         BoolOr,
         Corr,
@@ -50,23 +54,29 @@ class TestGeneralAggregate(PostgreSQLTestCase):
         cls.aggs = AggregateTestModel.objects.bulk_create(
             [
                 AggregateTestModel(
-                    boolean_field=True, char_field="Foo1", integer_field=0
+                    boolean_field=True,
+                    char_field="Foo1",
+                    text_field="Text1",
+                    integer_field=0,
                 ),
                 AggregateTestModel(
                     boolean_field=False,
                     char_field="Foo2",
+                    text_field="Text2",
                     integer_field=1,
                     json_field={"lang": "pl"},
                 ),
                 AggregateTestModel(
                     boolean_field=False,
                     char_field="Foo4",
+                    text_field="Text4",
                     integer_field=2,
                     json_field={"lang": "en"},
                 ),
                 AggregateTestModel(
                     boolean_field=True,
                     char_field="Foo3",
+                    text_field="Text3",
                     integer_field=0,
                     json_field={"breed": "collie"},
                 ),
@@ -87,6 +97,8 @@ class TestGeneralAggregate(PostgreSQLTestCase):
             (JSONBAgg("integer_field"), []),
             (StringAgg("char_field", delimiter=";"), ""),
         ]
+        if connection.features.has_bit_xor:
+            tests.append((BitXor("integer_field"), None))
         for aggregation, expected_result in tests:
             with self.subTest(aggregation=aggregation):
                 # Empty result with non-execution optimization.
@@ -118,6 +130,8 @@ class TestGeneralAggregate(PostgreSQLTestCase):
                 "<empty>",
             ),
         ]
+        if connection.features.has_bit_xor:
+            tests.append((BitXor("integer_field", default=0), 0))
         for aggregation, expected_result in tests:
             with self.subTest(aggregation=aggregation):
                 # Empty result with non-execution optimization.
@@ -330,6 +344,28 @@ class TestGeneralAggregate(PostgreSQLTestCase):
         )
         self.assertEqual(values, {"bitor": 0})
 
+    @skipUnlessDBFeature("has_bit_xor")
+    def test_bit_xor_general(self):
+        AggregateTestModel.objects.create(integer_field=3)
+        values = AggregateTestModel.objects.filter(
+            integer_field__in=[1, 3],
+        ).aggregate(bitxor=BitXor("integer_field"))
+        self.assertEqual(values, {"bitxor": 2})
+
+    @skipUnlessDBFeature("has_bit_xor")
+    def test_bit_xor_on_only_true_values(self):
+        values = AggregateTestModel.objects.filter(
+            integer_field=1,
+        ).aggregate(bitxor=BitXor("integer_field"))
+        self.assertEqual(values, {"bitxor": 1})
+
+    @skipUnlessDBFeature("has_bit_xor")
+    def test_bit_xor_on_only_false_values(self):
+        values = AggregateTestModel.objects.filter(
+            integer_field=0,
+        ).aggregate(bitxor=BitXor("integer_field"))
+        self.assertEqual(values, {"bitxor": 0})
+
     def test_bool_and_general(self):
         values = AggregateTestModel.objects.aggregate(booland=BoolAnd("boolean_field"))
         self.assertEqual(values, {"booland": False})
@@ -366,6 +402,12 @@ class TestGeneralAggregate(PostgreSQLTestCase):
         )
         self.assertEqual(values, {"stringagg": "Foo1;Foo2;Foo4;Foo3"})
 
+    def test_string_agg_default_output_field(self):
+        values = AggregateTestModel.objects.aggregate(
+            stringagg=StringAgg("text_field", delimiter=";"),
+        )
+        self.assertEqual(values, {"stringagg": "Text1;Text2;Text4;Text3"})
+
     def test_string_agg_charfield_ordering(self):
         ordering_test_cases = (
             (F("char_field").desc(), "Foo4;Foo3;Foo2;Foo1"),
@@ -628,6 +670,22 @@ class TestGeneralAggregate(PostgreSQLTestCase):
             inner_qs.values_list("integer_field", flat=True),
         )
 
+    def test_window(self):
+        self.assertCountEqual(
+            AggregateTestModel.objects.annotate(
+                integers=Window(
+                    expression=ArrayAgg("char_field"),
+                    partition_by=F("integer_field"),
+                )
+            ).values("integers", "char_field"),
+            [
+                {"integers": ["Foo1", "Foo3"], "char_field": "Foo1"},
+                {"integers": ["Foo1", "Foo3"], "char_field": "Foo3"},
+                {"integers": ["Foo2"], "char_field": "Foo2"},
+                {"integers": ["Foo4"], "char_field": "Foo4"},
+            ],
+        )
+
 
 class TestAggregateDistinct(PostgreSQLTestCase):
     @classmethod
