@@ -90,7 +90,7 @@ class SelectForUpdateTests(TransactionTestCase):
         generated SQL when select_for_update is invoked.
         """
         with transaction.atomic(), CaptureQueriesContext(connection) as ctx:
-            list(Person.objects.all().select_for_update())
+            list(Person.objects.select_for_update())
         self.assertTrue(self.has_for_update_sql(ctx.captured_queries))
 
     @skipUnlessDBFeature("has_select_for_update_nowait")
@@ -100,7 +100,7 @@ class SelectForUpdateTests(TransactionTestCase):
         generated SQL when select_for_update is invoked.
         """
         with transaction.atomic(), CaptureQueriesContext(connection) as ctx:
-            list(Person.objects.all().select_for_update(nowait=True))
+            list(Person.objects.select_for_update(nowait=True))
         self.assertTrue(self.has_for_update_sql(ctx.captured_queries, nowait=True))
 
     @skipUnlessDBFeature("has_select_for_update_skip_locked")
@@ -110,7 +110,7 @@ class SelectForUpdateTests(TransactionTestCase):
         generated SQL when select_for_update is invoked.
         """
         with transaction.atomic(), CaptureQueriesContext(connection) as ctx:
-            list(Person.objects.all().select_for_update(skip_locked=True))
+            list(Person.objects.select_for_update(skip_locked=True))
         self.assertTrue(self.has_for_update_sql(ctx.captured_queries, skip_locked=True))
 
     @skipUnlessDBFeature("has_select_for_no_key_update")
@@ -120,7 +120,7 @@ class SelectForUpdateTests(TransactionTestCase):
         select_for_update() is invoked.
         """
         with transaction.atomic(), CaptureQueriesContext(connection) as ctx:
-            list(Person.objects.all().select_for_update(no_key=True))
+            list(Person.objects.select_for_update(no_key=True))
         self.assertIs(self.has_for_update_sql(ctx.captured_queries, no_key=True), True)
 
     @skipUnlessDBFeature("has_select_for_update_of")
@@ -282,7 +282,16 @@ class SelectForUpdateTests(TransactionTestCase):
             )
         self.assertEqual(values, [{"born__name": self.city1.name}])
 
-    @skipUnlessDBFeature("has_select_for_update_nowait")
+    @skipUnlessDBFeature(
+        "has_select_for_update_of",
+        "supports_select_for_update_with_limit",
+    )
+    def test_for_update_of_with_exists(self):
+        with transaction.atomic():
+            qs = Person.objects.select_for_update(of=("self", "born"))
+            self.assertIs(qs.exists(), True)
+
+    @skipUnlessDBFeature("has_select_for_update_nowait", "supports_transactions")
     def test_nowait_raises_error_on_block(self):
         """
         If nowait is specified, we expect an error to be raised rather
@@ -303,7 +312,7 @@ class SelectForUpdateTests(TransactionTestCase):
         self.end_blocking_transaction()
         self.assertIsInstance(status[-1], DatabaseError)
 
-    @skipUnlessDBFeature("has_select_for_update_skip_locked")
+    @skipUnlessDBFeature("has_select_for_update_skip_locked", "supports_transactions")
     def test_skip_locked_skips_locked_rows(self):
         """
         If skip_locked is specified, the locked row is skipped resulting in
@@ -482,7 +491,7 @@ class SelectForUpdateTests(TransactionTestCase):
                     str(Person.objects.filter(name="foo").select_for_update().query),
                 )
 
-    @skipUnlessDBFeature("has_select_for_update")
+    @skipUnlessDBFeature("has_select_for_update", "supports_transactions")
     def test_for_update_requires_transaction(self):
         """
         A TransactionManagementError is raised
@@ -490,16 +499,16 @@ class SelectForUpdateTests(TransactionTestCase):
         """
         msg = "select_for_update cannot be used outside of a transaction."
         with self.assertRaisesMessage(transaction.TransactionManagementError, msg):
-            list(Person.objects.all().select_for_update())
+            list(Person.objects.select_for_update())
 
-    @skipUnlessDBFeature("has_select_for_update")
+    @skipUnlessDBFeature("has_select_for_update", "supports_transactions")
     def test_for_update_requires_transaction_only_in_execution(self):
         """
         No TransactionManagementError is raised
         when select_for_update is invoked outside of a transaction -
         only when the query is executed.
         """
-        people = Person.objects.all().select_for_update()
+        people = Person.objects.select_for_update()
         msg = "select_for_update cannot be used outside of a transaction."
         with self.assertRaisesMessage(transaction.TransactionManagementError, msg):
             list(people)
@@ -508,7 +517,7 @@ class SelectForUpdateTests(TransactionTestCase):
     def test_select_for_update_with_limit(self):
         other = Person.objects.create(name="Grappeli", born=self.city1, died=self.city2)
         with transaction.atomic():
-            qs = list(Person.objects.all().order_by("pk").select_for_update()[1:2])
+            qs = list(Person.objects.order_by("pk").select_for_update()[1:2])
             self.assertEqual(qs[0], other)
 
     @skipIfDBFeature("supports_select_for_update_with_limit")
@@ -519,7 +528,7 @@ class SelectForUpdateTests(TransactionTestCase):
         )
         with self.assertRaisesMessage(NotSupportedError, msg):
             with transaction.atomic():
-                list(Person.objects.all().order_by("pk").select_for_update()[1:2])
+                list(Person.objects.order_by("pk").select_for_update()[1:2])
 
     def run_select_for_update(self, status, **kwargs):
         """
@@ -590,7 +599,7 @@ class SelectForUpdateTests(TransactionTestCase):
         p = Person.objects.get(pk=self.person.pk)
         self.assertEqual("Fred", p.name)
 
-    @skipUnlessDBFeature("has_select_for_update")
+    @skipUnlessDBFeature("has_select_for_update", "supports_transactions")
     def test_raw_lock_not_available(self):
         """
         Running a raw query which can't obtain a FOR UPDATE lock raises
