@@ -99,7 +99,7 @@ class LiveServerTestCloseConnectionTest(LiveServerBase):
         self.assertIsNotNone(conn.connection)
         with self.urlopen("/model_view/") as f:
             # The server can access the database.
-            self.assertEqual(f.read().splitlines(), [b"jane", b"robert"])
+            self.assertCountEqual(f.read().splitlines(), [b"jane", b"robert"])
         # Wait for the server's request thread to close the connection.
         # A timeout of 0.1 seconds should be more than enough. If the wait
         # times out, the assertion after should fail.
@@ -127,18 +127,12 @@ class LiveServerTestCaseSetupTest(LiveServerBase):
             super().setUpClass()
         except RuntimeError:
             # LiveServerTestCase's change to ALLOWED_HOSTS should be reverted.
+            cls.doClassCleanups()
             cls.check_allowed_hosts(["testserver"])
         else:
             raise RuntimeError("Server did not fail.")
         cls.set_up_called = True
 
-    @classmethod
-    def tearDownClass(cls):
-        # Make tearDownClass() a no-op because setUpClass() was already cleaned
-        # up, and because the error inside setUpClass() was handled, which will
-        # cause tearDownClass() to be called when it normally wouldn't.
-        pass
-
     def test_set_up_class(self):
         self.assertIs(self.set_up_called, True)
 
@@ -179,7 +173,7 @@ class LiveServerViews(LiveServerBase):
         we can detect a content length from the response. This should be doable
         for all simple views and streaming responses where an iterable with
         length of one is passed. The latter follows as result of `set_content_length`
-        from https://github.com/python/cpython/blob/master/Lib/wsgiref/handlers.py.
+        from https://github.com/python/cpython/blob/main/Lib/wsgiref/handlers.py.
 
         If we cannot detect a content length we explicitly set the `Connection`
         header to `close` to notify the client that we do not actually support
@@ -295,7 +289,7 @@ class LiveServerViews(LiveServerBase):
 
 
 @override_settings(ROOT_URLCONF="servers.urls")
-class SingleTreadLiveServerViews(SingleThreadLiveServerTestCase):
+class SingleThreadLiveServerViews(SingleThreadLiveServerTestCase):
     available_apps = ["servers"]
 
     def test_closes_connection_with_content_length(self):
@@ -305,8 +299,8 @@ class SingleTreadLiveServerViews(SingleThreadLiveServerTestCase):
         Persistent connections require threading server.
         """
         conn = HTTPConnection(
-            SingleTreadLiveServerViews.server_thread.host,
-            SingleTreadLiveServerViews.server_thread.port,
+            SingleThreadLiveServerViews.server_thread.host,
+            SingleThreadLiveServerViews.server_thread.port,
             timeout=1,
         )
         try:
@@ -326,7 +320,7 @@ class LiveServerDatabase(LiveServerBase):
         Fixtures are properly loaded and visible to the live server thread.
         """
         with self.urlopen("/model_view/") as f:
-            self.assertEqual(f.read().splitlines(), [b"jane", b"robert"])
+            self.assertCountEqual(f.read().splitlines(), [b"jane", b"robert"])
 
     def test_database_writes(self):
         """
@@ -335,7 +329,7 @@ class LiveServerDatabase(LiveServerBase):
         with self.urlopen("/create_model_instance/"):
             pass
         self.assertQuerysetEqual(
-            Person.objects.all().order_by("pk"),
+            Person.objects.order_by("pk"),
             ["jane", "robert", "emily"],
             lambda b: b.name,
         )
@@ -349,7 +343,7 @@ class LiveServerPort(LiveServerBase):
         """
         TestCase = type("TestCase", (LiveServerBase,), {})
         try:
-            TestCase.setUpClass()
+            TestCase._start_server_thread()
         except OSError as e:
             if e.errno == errno.EADDRINUSE:
                 # We're out of ports, LiveServerTestCase correctly fails with
@@ -357,17 +351,12 @@ class LiveServerPort(LiveServerBase):
                 return
             # Unexpected error.
             raise
-        try:
-            # We've acquired a port, ensure our server threads acquired
-            # different addresses.
-            self.assertNotEqual(
-                self.live_server_url,
-                TestCase.live_server_url,
-                "Acquired duplicate server addresses for server threads: %s"
-                % self.live_server_url,
-            )
-        finally:
-            TestCase.tearDownClass()
+        self.assertNotEqual(
+            self.live_server_url,
+            TestCase.live_server_url,
+            f"Acquired duplicate server addresses for server threads: "
+            f"{self.live_server_url}",
+        )
 
     def test_specified_port_bind(self):
         """LiveServerTestCase.port customizes the server's port."""
@@ -377,16 +366,13 @@ class LiveServerPort(LiveServerBase):
         s.bind(("", 0))
         TestCase.port = s.getsockname()[1]
         s.close()
-        TestCase.setUpClass()
-        try:
-            self.assertEqual(
-                TestCase.port,
-                TestCase.server_thread.port,
-                "Did not use specified port for LiveServerTestCase thread: %s"
-                % TestCase.port,
-            )
-        finally:
-            TestCase.tearDownClass()
+        TestCase._start_server_thread()
+        self.assertEqual(
+            TestCase.port,
+            TestCase.server_thread.port,
+            f"Did not use specified port for LiveServerTestCase thread: "
+            f"{TestCase.port}",
+        )
 
 
 class LiveServerThreadedTests(LiveServerBase):
