@@ -129,7 +129,7 @@ class TestUtilsHtml(SimpleTestCase):
                     stripped = strip_tags(content)
                     elapsed = datetime.now() - start
                 self.assertEqual(elapsed.seconds, 0)
-                self.assertIn("Please try again.", stripped)
+                self.assertIn("Test string that has not been stripped.", stripped)
                 self.assertNotIn("<", stripped)
 
     def test_strip_spaces_between_tags(self):
@@ -211,6 +211,12 @@ class TestUtilsHtml(SimpleTestCase):
             with self.subTest(arg=arg):
                 self.assertEqual(json_script(arg, "test_id"), expected)
 
+    def test_json_script_without_id(self):
+        self.assertHTMLEqual(
+            json_script({"key": "value"}),
+            '<script type="application/json">{"key": "value"}</script>',
+        )
+
     def test_smart_urlquote(self):
         items = (
             ("http://öäü.com/", "http://xn--4ca9at.com/"),
